package edu.iris.dmc.ws;

import static org.junit.Assert.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import edu.iris.dmc.criteria.EventCriteria;
import edu.iris.dmc.event.model.Event;
import edu.iris.dmc.event.model.Magnitude;
import edu.iris.dmc.event.model.Origin;
import edu.iris.dmc.service.EventService;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.ServiceUtil;

public class EventServiceTest {

	EventService eventService;

	@Before
	public void setUp() throws FileNotFoundException, IOException {
		ServiceUtil serviceManager = ServiceUtil.getInstance();
		this.eventService = serviceManager.getEventService("http://service.iris.edu/fdsnws/event/1/");
	}

	//@Test(expected = NoDataFoundException.class)
	public void noDataFoundException() throws Exception {
		EventCriteria criteria = new EventCriteria();
		// no magnitudes of this value, therefore 404
		criteria.setMinimumMagnitude(11.0);
		this.eventService.fetch(criteria);
	}

	@Test
	public void fetch() throws Exception {

		EventCriteria criteria = new EventCriteria();
		criteria.setEventId("3279407");
		criteria.includeAllMagnitudes(true);

		String expectedEventId = "smi:service.iris.edu/fdsnws/event/1/query?eventid=3279407";

		List<Event> events = eventService.fetch(criteria);
		assertNotNull(events);
		assertTrue(!events.isEmpty());


		Event event = null;
		for (Event e : events) {System.out.println(e.getPublicId()+"///////");
			if (expectedEventId.equals(e.getPublicId())) {
				event = e;
			}
			System.out.println(e.getPreferredMagnitude()+"   "+e.getPreferredOrigin());
		}

		assertNotNull(event);

		String preferredOriginId = "smi:service.iris.edu/fdsnws/event/1/query?originid=9016956";
		Origin preferredOrigin = event.getPreferredOrigin();
		
		System.out.println(preferredOrigin.getPublicId());

		assertEquals(preferredOriginId, preferredOrigin.getPublicId());
	}

	
	@Test
	public void fetchArrivals() throws Exception {

		EventCriteria criteria = new EventCriteria();
		criteria.setEventId("3326785");
		criteria.includeArrivals(true);

		String expectedEventId = "smi:service.iris.edu/fdsnws/event/1/query?eventid=3326785";

		List<Event> events = eventService.fetch(criteria);
		assertNotNull(events);
		assertTrue(!events.isEmpty());


		Event event = null;
		for (Event e : events) {
			if (expectedEventId.equals(e.getPublicId())) {
				event = e;
			}
			e.getPublicId();
			e.getType();
			e.getPreferredOrigin();
			for (Magnitude magnitude : e.getMagnitudes()) {
				magnitude.getPublicId();
				magnitude.getType();
				magnitude.getAuthor();
				magnitude.getValue();
				//System.out.println(magnitude.getPublicId()+"  "+magnitude.getType()+"  "+magnitude.getAuthor());
			}
			for (Origin origin : e.getOrigins()) {
				origin.getPublicId();
				origin.getTime();
				origin.getLatitude();
				origin.getLongitude();
				origin.getDepth();
				//System.out.println(origin.getContributorEventId()+"  "+origin.getContributorOriginId()+"  "+origin.getCatalog()+"  "+origin.getContributor());
			}
		}

		assertNotNull(event);

		String preferredOriginId = "smi:service.iris.edu/fdsnws/event/1/query?originid=7730536";
		Origin preferredOrigin = event.getPreferredOrigin();

		assertEquals(preferredOriginId, preferredOrigin.getPublicId());
	}
	
	//@Test
	public void eventId() throws Exception {

		EventCriteria criteria = new EventCriteria();
		criteria.setEventId("454963");

		String expectedEventId = "smi:service.iris.edu/fdsnws/event/1/query?eventid=454963";

		List<Event> events = eventService.fetch(criteria);
		assertNotNull(events);
		assertTrue(!events.isEmpty());


		Event event = null;
		for (Event e : events) {
			if (expectedEventId.equals(e.getPublicId())) {
				event = e;
			}
			e.getPublicId();
			e.getType();
			e.getPreferredOrigin();
			for (Magnitude magnitude : e.getMagnitudes()) {
				magnitude.getPublicId();
				magnitude.getType();
				magnitude.getAuthor();
				magnitude.getValue();
				//System.out.println(magnitude.getPublicId()+"  "+magnitude.getType()+"  "+magnitude.getAuthor());
			}
			for (Origin origin : e.getOrigins()) {
				origin.getPublicId();
				origin.getTime();
				origin.getLatitude();
				origin.getLongitude();
				origin.getDepth();
				System.out.println(origin.getTime().getTime() % 1000);
				int n = (int) origin.getTime().getTime() % 1000;System.out.println(n);
				n = n<0 ? n+1000 : n;
				//System.out.println(origin.getPublicId()+"    "+origin.getTime().toString()+"   "+n);
			}
		}

		assertNotNull(event);
		
		assertNotNull(event.getFlinnEngdahlRegionCode());
		assertEquals(55,event.getFlinnEngdahlRegionCode().intValue());
		assertNotNull(event.getFlinnEngdahlRegionName());
		assertEquals("NEAR COAST OF JALISCO, MEXICO", event.getFlinnEngdahlRegionName());
		

		String preferredOriginId = "smi:www.iris.edu/ws/event/query?originId=958881";
		Origin preferredOrigin = event.getPreferredOrigin();
		//System.out.println("----"+preferredOrigin.getTime());

		assertEquals(preferredOriginId, preferredOrigin.getPublicId());
	}
	
	///@Test
	public void load() throws Exception {
		File theFile = new File(
				"/Users/yazan/iris/test/eventLoadedTestFile.xml");
		InputStream inputStream = new FileInputStream(theFile);
		List<Event> events = this.eventService.load(inputStream);

		assertNotNull(events);
		assertTrue(!events.isEmpty());

		Event event = null;
		String expectedEventId = "smi:www.iris.edu/ws/event/query?eventId=3318610";
		for (Event e : events) {
			if (expectedEventId.equalsIgnoreCase(e.getPublicId())) {
				event = e;
			}
		}

		assertNotNull(event);

		String preferredOriginId = "smi:www.iris.edu/ws/event/query?originId=7730536";
		Origin preferredOrigin = event.getPreferredOrigin();

		assertEquals(preferredOriginId, preferredOrigin.getPublicId());
	}

	//@Test
	public void stream() throws Exception {

		File theFile = new File("/Users/yazan/iris/test/event_test_input.xml");
		OutputStream out = new FileOutputStream(theFile);
		assertTrue(theFile.exists());
		EventCriteria criteria = new EventCriteria();
		criteria.setMinimumMagnitude(8D);

		this.eventService.stream(out, criteria);
		out.flush();
		out.close();

		assertEquals(246150, theFile.length());

	}
}
