package edu.iris.dmc.ws;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.OutputLevel;
import edu.iris.dmc.criteria.StationCriteria;
import edu.iris.dmc.fdsn.station.model.Channel;
import edu.iris.dmc.fdsn.station.model.Network;
import edu.iris.dmc.fdsn.station.model.Station;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.ServiceNotSupportedException;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.StationIterator;
import edu.iris.dmc.service.StationService;
import edu.iris.dmc.service.UnauthorizedAccessException;

public class Main {

	private StationService stationService;

	public void fetch(String network, String station, String channel,
			String location, Date start, Date end) throws NoDataFoundException,
			CriteriaException, IOException, ServiceNotSupportedException {

		StationCriteria criteria = new StationCriteria();
		criteria.addNetwork(network).addStation(station).addChannel(channel)
				.addLocation(location);
		criteria.setLevel(OutputLevel.CHANNEL);
		criteria.setStartTime(start);
		criteria.setEndTime(end);

		List<Network> networks = stationService.fetch(criteria);		
		
		for (Network n : networks) {
			System.out.println(n);
			for (Station s : n.getStations()) {
				System.out.println(s.toString());
				for (Channel c : s.getChannels()) {
					System.out.println(c.toString());
				}
			}
		}
	}

	public void iterate(String network, String station, String channel,
			String location, Date start, Date end) throws NoDataFoundException,
			CriteriaException, IOException, ServiceNotSupportedException {
		try {
			StationIterator si = stationService
			//Iterator<Station> si = stationService
					//.iterate("http://service.iris.edu/fdsnws/station/1/query?net=IU&sta=ANMO&level=resp&cha=BHZ&loc=--&format=text");
					.iterate("http://service.iris.edu/fdsnws/station/1/query?net=GE&sta=BOAB&channel=B*&location=--&level=channel&start=2010-01-01&end=2010-01-06");
					//.iterate("http://service.iris.edu/fdsnws/station/1/query?net=AK&sta=F*&cha=B?Z&level=channel&loc=--&format=text");
					//.iterate("http://service.iris.edu/fdsnws/station/1/query?net=7D&sta=FN01A&cha=HDH&start=2012-06-01&end=2012-06-02");
			while (si.hasNext()) {
				Station s = si.next();
			}
			
			if (si != null) {
				System.out.printf("myothertext\n");
				si.close();
			}
			
			while (si.hasNext()) {
				Station s = si.next();
				System.out.println(s.getCode());
				System.out.printf("\tStation : %s with %d channels\n",
						s.getCode(), s.getChannels().size());
				for (Channel c : s.getChannels()) {
					System.out.printf("\t\tChannel: %s on: %s   off:%s\n",
							c.getCode(), c.getStartDate(), c.getEndDate());
				}
			}

		} catch (FileNotFoundException e) {
			// No file found
			e.printStackTrace();
		} catch (IOException e) {
			// Something IO didn't work
			e.printStackTrace();
		} catch (NoDataFoundException e) {
			// The query didn't find any data
			e.printStackTrace();
		} catch (CriteriaException e) {
			// Something went wrong with the chosen criteria
			e.printStackTrace();
		} catch (ServiceNotSupportedException e) {
			e.printStackTrace();
		}
	}

	public void load(InputStream inputStream) {
		try {
			List<Network> networks = stationService.load(inputStream);
			for (Network n : networks) {
				System.out.println(n.getCode());
				for (Station s : n.getStations()) {
					System.out.println(s.getCode());
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void stream(String network, String station, String channel,
			String location, Date start, Date end) throws NoDataFoundException,
			CriteriaException, IOException, ServiceNotSupportedException {
		OutputStream out = null;
		try {
			StationCriteria stationCriteria = new StationCriteria();
			stationCriteria.addNetwork(network).addStation(station).addChannel(channel)
				.addLocation(location);
			stationCriteria.setLevel(OutputLevel.CHANNEL);
			stationCriteria.setStartTime(start);
			stationCriteria.setEndTime(end);
	
			out = new FileOutputStream(new File(
					"\\Users\\rtweekly\\Desktop\\station-test.xml"));

			//stationCriteria.addNetwork("AK");
			//stationCriteria.addStation("F*");
			//stationCriteria.addChannel("??Z");

			// fetch the data
			// List<Network> networks =
			// stationService.fetch(stationCriteria, OutputLevel.CHANNEL);

			stationService.stream(out, stationCriteria);

			System.out.println(network);
			System.out.println("\t" + station);
			System.out.println("\t\t\t" + channel);
		} catch (UnauthorizedAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			if (out != null) {
				out.close();
			}
		}

	}

	public Main() {
		ServiceUtil serviceManager = ServiceUtil.getInstance();
		serviceManager.setAppName("Examples");
		stationService = serviceManager.getStationService();
	}

	public static void main(String[] args) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss Z");
		Main example = new Main();
		try {

			example.fetch("AF", "BLWY", "BH?", "--",
				sdf.parse("2008-10-31 23:00:00 UTC"),
				sdf.parse("2014-01-01 00:00:00 UTC"));
			//example.iterate("IU", "ANMO", "BHZ", "00",
					//sdf.parse("2010-02-27 06:30:00"),
					//sdf.parse("2010-02-27 10:30:00"));
			//example.stream("7D", "FNO1A", "*", "--", 
					//sdf.parse("2012-06-01 00:00:00"), 
					//sdf.parse("2012-06-02 23:59:59"));
			//example.stream("AF", "BLWY", "BH?", "--",
					//sdf.parse("2008-10-31 00:00:00"),
					//sdf.parse("2014-01-01 00:00:00"));
			//example.load(new FileInputStream(new File(
					//"/Users/rtweekly/Desktop/station-test.xml")));
		} catch (NoDataFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CriteriaException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ServiceNotSupportedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
