package edu.iris.dmc.ws;

import static org.junit.Assert.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.junit.Before;
import org.junit.Test;

import edu.iris.dmc.sacpz.model.Pole;
import edu.iris.dmc.sacpz.model.Sacpz;
import edu.iris.dmc.sacpz.model.Zero;
import edu.iris.dmc.service.EventService;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.SacpzService;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.StationService;
import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.EventCriteria;
import edu.iris.dmc.criteria.OutputLevel;
import edu.iris.dmc.criteria.SacpzCriteria;
import edu.iris.dmc.criteria.StationCriteria;
import edu.iris.dmc.event.model.Event;
import edu.iris.dmc.event.model.Origin;
import edu.iris.dmc.fdsn.station.model.Channel;
import edu.iris.dmc.fdsn.station.model.Filter;
import edu.iris.dmc.fdsn.station.model.Network;
import edu.iris.dmc.fdsn.station.model.PolesZeros;

import edu.iris.dmc.fdsn.station.model.Station;


public class SacpzServiceTest {

	SacpzService sacpzService;

	@Before
	public void setUp() throws FileNotFoundException, IOException {
		ServiceUtil serviceManager = ServiceUtil.getInstance();
		serviceManager.setAppName("Yazan Testing App");
		this.sacpzService = serviceManager.getSacpzService("http://service.iris.edu/irisws/sacpz/1/");
	}


	@Test
	public void poles() throws Exception {

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'H:m:s");
		sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date startDate = sdf.parse("2000-06-13T23:58:00");
		Date endDate = sdf.parse("2000-06-14T00:02:00");

		SacpzCriteria sc = new SacpzCriteria();
		sc.addNetwork("XE");
		sc.addChannel("BHN");
		sc.addLocation("01");
		sc.addStation("DH1");
		sc.setStartTime(startDate);
		sc.setEndTime(endDate);

		System.out.println(sc.toUrlParams());

		List<Sacpz> ls = sacpzService.fetch(sc);

		for(Sacpz sacpz:ls){
			for(Zero z:sacpz.getZeros()){
				System.out.println(z.getReal()+"  "+z.getImaginary());
			}
			for(Pole p:sacpz.getPoles()){
				System.out.println(p.getReal()+"  "+p.getImaginary());
			}
			System.out.println(sacpz.getInclination());
			System.out.println("*****************");
		}
		assertEquals(1, ls.size());
/*
		Network iu = ls.get(0);

		assertEquals("IU", iu.getCode());
		assertEquals(1, iu.getStations().size());
		assertEquals(1, iu.getSelectedNumberStations());
		assertEquals(91, iu.getTotalNumberStations());

		ChannelEpoch theChannelEpoch = null;

		for (Station station : iu.getStations()) {
			for (StationEpoch stationEpoch : station.getEpochs()) {
				System.out.println("///////////"+stationEpoch.getNetworkIdentifier());
				for (Channel channel : stationEpoch.getChannels()) {
					System.out.println(channel.toString());
					for (ChannelEpoch channelEpoch : channel.getEpochs()) {
						if (startDate.equals(channelEpoch.getStartDate())
								&& endDate.equals(channelEpoch.getEndDate())) {
							theChannelEpoch = channelEpoch;
							System.out.println(channelEpoch.getFlags());
							System.out.println("00000"+channelEpoch.getCalibrationUnit());
						}
					}
				}
			}
		}

		assertNotNull(theChannelEpoch);
		assertEquals(20, theChannelEpoch.getSampleRate().intValue());

		Sensitivity channelEpochSensitivity = theChannelEpoch.getSensitivity();

		assertNotNull(channelEpochSensitivity);
		assertEquals("M/S",
				channelEpochSensitivity.getUnits());
		
		assertEquals("Velocity in Meters Per Second",channelEpochSensitivity.getUnitDescription());

		double frequency = channelEpochSensitivity.getFrequency() * 100;
		assertEquals(2, (int) frequency);
		assertEquals(864730000, channelEpochSensitivity.getValue().intValue());

		assertEquals(6, theChannelEpoch.getResponse().size());

		Response theResponse = null;
		for (Response response : theChannelEpoch.getResponse()) {
			if (response.getStage() == 1) {
				theResponse = response;
			}
		}
		assertNotNull(theResponse);


		assertEquals(1, theResponse.getFilters().size());

		Filter filter = theResponse.getFilters().get(0);

		assertEquals(53, filter.getType().intValue());


		PolesZeros polesZeros = (PolesZeros) filter;
		assertEquals(5, polesZeros.getPoles().size());
		assertEquals(2, polesZeros.getZeros().size());
		
		System.out.println(polesZeros.getTransferFunction());
*/
	}

}
