package edu.iris.dmc.ws;

import static org.junit.Assert.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.TimeZone;

import org.junit.Before;
import org.junit.Test;

import edu.iris.dmc.service.EventService;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.StationService;
import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.EventCriteria;
import edu.iris.dmc.criteria.OutputLevel;
import edu.iris.dmc.criteria.StationCriteria;
import edu.iris.dmc.event.model.Event;
import edu.iris.dmc.event.model.Origin;
import edu.iris.dmc.fdsn.station.model.Channel;
import edu.iris.dmc.fdsn.station.model.Filter;
import edu.iris.dmc.fdsn.station.model.Network;
import edu.iris.dmc.fdsn.station.model.PolesZeros;
import edu.iris.dmc.fdsn.station.model.Response;
import edu.iris.dmc.fdsn.station.model.ResponseStage;
import edu.iris.dmc.fdsn.station.model.Sensitivity;
import edu.iris.dmc.fdsn.station.model.Station;

public class StationServiceTest {

	StationService stationService;

	@Before
	public void setUp() throws FileNotFoundException, IOException {
		ServiceUtil serviceManager = ServiceUtil.getInstance();
		serviceManager.setAppName("Yazan Testing App");
		this.stationService = serviceManager
				.getStationService("http://service.iris.edu/fdsnwsbeta/station/1/");
	}

	@Test(expected = NoDataFoundException.class)
	public void noDataFoundException() throws Exception {
		StationCriteria criteria = new StationCriteria();
		// no magnitudes of this value, therefore 404
		criteria.addNetwork("IU");
		criteria.addStation("dummy");
		this.stationService.fetch(criteria, OutputLevel.STATION);
	}

	@Test
	public void criteriaLevelTest() throws Exception {
		StationCriteria criteria = new StationCriteria();
		criteria.addNetwork("IU");
		criteria.addStation("ANMO");

		Iterator<Station> it = stationService.iterateStations(criteria,
				OutputLevel.STATION);
		// networks = this.stationService.fetch(criteria);
		while (it.hasNext()) {
			Station station = it.next();

			System.out.println("///////"+station.getNetwork()+"   "+station.getNetwork().getRootDocument());
		}

		// more testing needed here
	}

    @Test
	public void poles() throws Exception {

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'H:m:s");
		sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date startDate = sdf.parse("1995-07-14T00:00:00");
		Date endDate = sdf.parse("1998-10-26T20:00:00");

		StationCriteria sc = new StationCriteria();
		sc.addNetwork("IU");
		sc.addChannel("BHZ");
		sc.addLocation("--");
		sc.addStation("ANMO");

		System.out.println(sc.toUrlParams());

		List<Network> ls = stationService.fetch(sc, OutputLevel.RESPONSE);

		assertEquals(1, ls.size());

		Network iu = ls.get(0);

		assertEquals("IU", iu.getCode());System.out.println(iu.getStations().size());
		//assertEquals(2, iu.getStations().size());
		assertEquals(2, iu.getSelectedNumberStations().intValue());
		assertEquals(264, iu.getTotalNumberStations().intValue());

		Channel theChannelEpoch = null;

		for (Station station : iu.getStations()) {

			for (Channel channel : station.getChannels()) {
				System.out.println(channel.toString());

				if (startDate.equals(channel.getStartDate())
						&& endDate.equals(channel.getEndDate())) {
					theChannelEpoch = channel;
					System.out.println("00000" + channel.getCalibrationUnits());
				}

			}

		}

		assertNotNull(theChannelEpoch);
		assertEquals(20, theChannelEpoch.getSampleRate().getValue());

		Sensitivity channelEpochSensitivity = theChannelEpoch.getResponse()
				.getInstrumentSensitivity();

		assertNotNull(channelEpochSensitivity);
		assertEquals("M/S", channelEpochSensitivity.getInputUnits().getName());

		assertEquals("Velocity in Meters Per Second", channelEpochSensitivity
				.getInputUnits().getDescription());

		double frequency = channelEpochSensitivity.getFrequency() * 100;
		assertEquals(2, (int) frequency);


		assertEquals(6, theChannelEpoch.getResponse().getStage().size());

		Response theResponse = theChannelEpoch.getResponse();

		for (ResponseStage stage : theResponse.getStage()) {
			if (stage.getNumber().intValue() == 1) {
				stage.getPolesZeros();
			}
		}
		assertNotNull(theResponse);

		/*
		 * assertEquals(1, theResponse.getFilters().size());
		 * 
		 * Filter filter = theResponse.getFilters().get(0);
		 * 
		 * assertEquals(53, filter.getType().intValue());
		 * 
		 * PolesZeros polesZeros = (PolesZeros) filter; assertEquals(5,
		 * polesZeros.getPoles().size()); assertEquals(2,
		 * polesZeros.getZeros().size());
		 * 
		 * System.out.println(polesZeros.getTransferFunction());
		 */

	}
	/*
	 * @Test public void timeWindow() throws Exception {
	 * 
	 * StationCriteria sc = new StationCriteria(); sc.addNetwork("IU");
	 * sc.addChannel("BHZ"); sc.addLocation("--");
	 * 
	 * SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
	 * sdf.setTimeZone(TimeZone.getTimeZone("GMT")); Date startDate =
	 * sdf.parse("2010-01-01"); Date endDate = sdf.parse("2011-10-10");
	 * 
	 * sc.setStartTime(startDate); sc.setEndTime(endDate);
	 * 
	 * System.out.println(sc.toUrlParams());
	 * 
	 * List<Network> ls = stationService.fetch(sc, OutputLevel.CHANNEL);
	 * 
	 * assertNotNull(ls); assertEquals(1, ls.size());
	 * 
	 * Network iu = ls.get(0);
	 * 
	 * assertEquals("IU", iu.getCode()); assertEquals(5,
	 * iu.getStations().size());
	 * 
	 * }
	 * 
	 * @Test(expected = CriteriaException.class) public void
	 * timeWindowWithException() throws Exception {
	 * 
	 * StationCriteria sc = new StationCriteria(); sc.addNetwork("IU");
	 * sc.addChannel("BHZ"); sc.addLocation("--");
	 * 
	 * SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
	 * sdf.setTimeZone(TimeZone.getTimeZone("GMT")); Date startDate =
	 * sdf.parse("2010-01-01"); Date endDate = sdf.parse("2011-10-10");
	 * 
	 * sc.setStartTime(startDate); // sc.setEndTime(endDate);
	 * 
	 * System.out.println(sc.toUrlParams());
	 * 
	 * List<Network> ls = stationService.fetch(sc, OutputLevel.CHANNEL);
	 * 
	 * assertNotNull(ls); assertEquals(1, ls.size());
	 * 
	 * Network iu = ls.get(0);
	 * 
	 * assertEquals("IU", iu.getCode()); assertEquals(5,
	 * iu.getStations().size());
	 * 
	 * }
	 * 
	 * // @Test(expected = NoDataFoundException.class) public void anotherTest()
	 * throws Exception {
	 * 
	 * StationCriteria sc = new StationCriteria(); sc.addNetwork("DW");
	 * sc.addChannel("BHZ"); sc.addLocation("--");
	 * 
	 * SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss");
	 * sdf.setTimeZone(TimeZone.getTimeZone("GMT")); Date startDate =
	 * sdf.parse("2011-11-08T08:00:00"); Date endDate =
	 * sdf.parse("2011-11-08T09:00:00");
	 * 
	 * // sc.setTimeWindow(startDate, endDate); sc.setStartBefore(endDate);
	 * sc.setEndAfter(startDate);
	 * 
	 * System.out.println(sc.toUrlParams());
	 * 
	 * List<Network> ls = stationService.fetch(sc, OutputLevel.CHANNEL);
	 * System.out.println("Found  " + ls.size() + " stations"); for (Network n :
	 * ls) { for (Station s : n.getStations()) { StringBuffer sb = new
	 * StringBuffer();
	 * 
	 * for (StationEpoch se : s.getEpochs()) { for (Channel c :
	 * se.getChannels()) { sb.append(s.getNetwork().getCode() + " " +
	 * s.getCode() + " " + c.getCode() + " " + c.getLocation());
	 * 
	 * }
	 * 
	 * } System.out.println(sb.toString()); } } }
	 * 
	 * @Test public void fetch() throws Exception {
	 * 
	 * StationCriteria criteria = new StationCriteria();
	 * criteria.addNetwork("IU"); criteria.addStation("ANMO");
	 * 
	 * criteria.addLocation("00"); String expectedStationCode = "ANMO";
	 * 
	 * List<Network> ls = stationService.fetch(criteria, OutputLevel.CHANNEL);
	 * assertNotNull(ls); assertTrue(!ls.isEmpty());
	 * 
	 * Station station = null; for (Network n : ls) { for (Station s :
	 * n.getStations()) { if (expectedStationCode.equalsIgnoreCase(s.getCode()))
	 * { station = s; }
	 * 
	 * for (StationEpoch se : s.getEpochs()) { for (Channel ch :
	 * se.getChannels()) { assertTrue(ch.getLocation().equals("00")); for
	 * (ChannelEpoch ce : ch.getEpochs()) { // System.out.println(ce); for
	 * (Response resp : ce.getResponse()) { int stage = resp.getStage();
	 * Sensitivity stageSensitvity = resp .getStageSensitivity(); List<Filter>
	 * filters = resp.getFilters(); } } } } } }
	 * 
	 * assertNotNull(station);
	 * 
	 * assertEquals(5, station.getEpochs().size());
	 * 
	 * }
	 * 
	 * // @Test public void load() throws Exception { File theFile = new File(
	 * "/Users/yazan/iris/test/stationLoadedTestFile.xml"); InputStream
	 * inputStream = new FileInputStream(theFile); List<Network> ls =
	 * this.stationService.load(inputStream);
	 * 
	 * assertNotNull(ls); assertTrue(!ls.isEmpty());
	 * 
	 * String expectedStationCode = "ANMO";
	 * 
	 * Station station = null; for (Network n : ls) { for (Station s :
	 * n.getStations()) { if (expectedStationCode.equalsIgnoreCase(s.getCode()))
	 * { station = s; }
	 * 
	 * for (StationEpoch se : s.getEpochs()) { for (Channel ch :
	 * se.getChannels()) { System.out.println(";;;;;;;"); for (ChannelEpoch ce :
	 * ch.getEpochs()) { System.out.println(ce); } } } } }
	 * 
	 * assertNotNull(station);
	 * 
	 * assertEquals(6, station.getEpochs().size()); }
	 * 
	 * // @Test public void stream() throws Exception {
	 * 
	 * File theFile = new File("/Users/yazan/iris/test/station_test_input.xml");
	 * OutputStream out = new FileOutputStream(theFile);
	 * assertTrue(theFile.exists()); StationCriteria criteria = new
	 * StationCriteria(); criteria.addNetwork("IU");
	 * criteria.addStation("ANMO");
	 * 
	 * this.stationService.stream(out, criteria); out.flush(); out.close();
	 * 
	 * assertEquals(3994, theFile.length());
	 * 
	 * }
	 */
}
