package edu.iris.dmc.ws;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.junit.Before;
import org.junit.Test;

import edu.iris.dmc.criteria.WaveformCriteria;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.WaveformService;
import edu.iris.dmc.timeseries.model.Segment;
import edu.iris.dmc.timeseries.model.Timeseries;
import edu.iris.dmc.timeseries.model.Segment.Type;

public class UnmergedWaveFormTest {
	WaveformService waveFormService;

	@Before
	public void setUp() throws FileNotFoundException, IOException {
		ServiceUtil serviceManager = ServiceUtil.getInstance();
		this.waveFormService = serviceManager.getWaveformService();
	}

	@Test
	public void fetch() throws Exception {
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date a = dfm.parse("2011-10-14T00:00:00:000");
		Date b = dfm.parse(" 2011-10-14T19:59:59:981");
		
		Date c = dfm.parse("2011-10-14T20:00:00:000");
		Date d = dfm.parse(" 2011-10-15T00:00:00:000");

		WaveformCriteria criteria = new WaveformCriteria();

		criteria.add("IU", "ANMO", "10", "BHZ", a, b);
		criteria.add("IU", "ANMO", "10", "BHZ", c, d);
		
		criteria.makeDistinctRequests(true);

		System.out.println(criteria.toUrlParams());

		List<Timeseries> timeSeriesCollection = waveFormService.fetch(criteria);
		assertNotNull(timeSeriesCollection);
		assertTrue(!timeSeriesCollection.isEmpty());
		
		assertEquals(2, timeSeriesCollection.size());

		Timeseries timeseries = null;
		for (Timeseries e : timeSeriesCollection) {
			if ("IU".equalsIgnoreCase(e.getNetworkCode())) {System.out.println("/////");
				timeseries = e;

				for (Segment s : e.getSegments()) {

					System.out.println("Sample Count: " + s.getSampleCount());
					if (s.getType() == Type.INTEGER) {
						assertNotNull(s.getIntData());
						assertEquals(s.getIntData().size(), s.getSampleCount());
					} else if (s.getType() == Type.FLOAT) {
						assertNotNull(s.getFloatData());
						assertEquals(s.getFloatData().size(),
								s.getSampleCount());
					} else if (s.getType() == Type.DOUBLE) {
						assertNotNull(s.getDoubleData());
						assertEquals(s.getDoubleData().size(),
								s.getSampleCount());
					} else if (s.getType() == Type.SHORT) {
						assertNotNull(s.getShortData());
						assertEquals(s.getShortData().size(),
								s.getSampleCount());
					}
				}
			}
		}

		assertNotNull(timeseries);

	}
}
