package edu.iris.dmc.ws;

import static java.net.HttpURLConnection.HTTP_OK;
import static java.util.logging.Level.FINER;
import static java.util.logging.Level.WARNING;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.Authenticator;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.PasswordAuthentication;
import java.net.ProtocolException;
import java.net.URL;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import edu.iris.dmc.criteria.Criteria;
import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.WaveformCriteria;
import edu.iris.dmc.seedcodec.CodecException;
import edu.iris.dmc.seedcodec.UnsupportedCompressionType;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.ServiceNotSupportedException;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.UnauthorizedAccessException;
import edu.iris.dmc.timeseries.model.Timeseries;
import edu.iris.dmc.timeseries.model.Util;
import edu.iris.dmc.ws.util.StringUtil;
import edu.sc.seis.seisFile.mseed.Blockette;
import edu.sc.seis.seisFile.mseed.Blockette1001;
import edu.sc.seis.seisFile.mseed.Btime;
import edu.sc.seis.seisFile.mseed.ControlHeader;
import edu.sc.seis.seisFile.mseed.DataBlockette;
import edu.sc.seis.seisFile.mseed.DataHeader;
import edu.sc.seis.seisFile.mseed.DataRecord;
import edu.sc.seis.seisFile.mseed.SeedFormatException;
import edu.sc.seis.seisFile.mseed.SeedRecord;

public class WaveformService {

	public static void main1(String args[]) {

		String loggerName = "edu.iris";
		Logger log = Logger.getLogger(loggerName);
		ConsoleHandler handler = new ConsoleHandler();
		handler.setLevel(Level.ALL);
		log.addHandler(handler);
		log.setLevel(Level.ALL);
		log.fine("just testing");

		HttpURLConnection connection = null;
		InputStream inputStream = null;

		List<Timeseries> collection = new ArrayList<Timeseries>();
		try {

			URL url = new URL("http://service.iris.edu/fdsnws/dataselect/query");

			connection = (HttpURLConnection) url.openConnection();

			connection.setRequestProperty("Content-Type",
					"application/x-www-form-urlencoded");
			connection.setRequestMethod("GET");
			connection.setDoOutput(true);
			connection.setDoInput(true);
			connection.setUseCaches(false);
			connection.setAllowUserInteraction(false);
			connection.connect();

			OutputStream outputStream = connection.getOutputStream();
			OutputStreamWriter writer = new OutputStreamWriter(outputStream);

			writer.write("IU ANMO 10 BHZ 2010-02-27T06.30.00 2010-02-28T06.20.00");
			writer.close();
			outputStream.close();

			int responseCode = connection.getResponseCode();
			inputStream = responseCode != HTTP_OK ? connection.getErrorStream()
					: connection.getInputStream();
			switch (responseCode) {
			case 404:
				break;
			case 204:
				break;
			case 400:
				break;
			case 500:
				break;
			case 200:
				BufferedInputStream data = new BufferedInputStream(inputStream);
				DataInputStream dis = new DataInputStream(data);

				List<DataRecord> list = new ArrayList<DataRecord>();
				while (true) {
					try {
						SeedRecord sr = SeedRecord.read(dis);
						if (sr instanceof DataRecord) {
							list.add((DataRecord) sr);
						}
					} catch (EOFException eoe) {
						eoe.printStackTrace();
						break;
					} catch (IOException e) {
						e.printStackTrace();
					}
				}

				if (!list.isEmpty()) {
					for (DataRecord dr : list) {
						byte microseconds = 0;
						Blockette[] bs = dr.getBlockettes(1001);
						if (bs.length > 0) {
							Blockette1001 b1001 = (Blockette1001) bs[0];
							microseconds = b1001.getMicrosecond();
						}

						if (dr.getBlockettes(1000).length != 0) {
							// ControlHeader
							DataHeader header = (DataHeader) dr
									.getControlHeader();
							String network = header.getNetworkCode();
							String station = header.getStationIdentifier();
							String location = header.getLocationIdentifier();
							String channel = header.getChannelIdentifier();

							Timestamp startTime = Util.toTime(
									header.getStartBtime(),
									header.getActivityFlags(),
									header.getTimeCorrection(), microseconds);

							Timeseries timeseries = new Timeseries(network,
									station, location, channel);
							int index = collection.indexOf(timeseries);
							if (index > -1) {
								timeseries = collection.get(index);
							} else {

								collection.add(timeseries);
							}

							try {
								timeseries.add(startTime, dr);
							} catch (UnsupportedCompressionType e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (CodecException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							// System.out.println(startTime);
						} else {
							// :TODO throw exception
						}
					}
				}

				break;
			default:
				throw new IOException(connection.getResponseMessage());
			}

		} catch (SeedFormatException e) {
			e.printStackTrace();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (inputStream != null) {
				try {
					inputStream.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			if (connection != null) {
				connection.disconnect();
			}
		}

	}

	public static void main(String[] args) {
		ServiceUtil serviceUtil = ServiceUtil.getInstance();
		edu.iris.dmc.service.WaveformService waveFormService = serviceUtil
				.getWaveformService();
		WaveformCriteria criteria = new WaveformCriteria();
		SimpleDateFormat sdf = new SimpleDateFormat("yy-mm-dd hh:mm:ss");
		try {
		criteria.add("IU", "ANMO", "00", "BHZ", sdf.parse("2010-02-27 06:30:00"), sdf.parse("2010-02-27 10:30:00"));
		FileOutputStream os;
		
			os = new FileOutputStream("/Users/yazan/test.mseed");

			waveFormService.stream(os, criteria);
			os.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoDataFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CriteriaException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ServiceNotSupportedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnauthorizedAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
