package edu.iris.dmc.ws;

import static org.junit.Assert.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.junit.Before;
import org.junit.Test;

import edu.iris.dmc.criteria.WaveformCriteria;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.WaveformService;
import edu.iris.dmc.timeseries.model.Segment;
import edu.iris.dmc.timeseries.model.Segment.Type;
import edu.iris.dmc.timeseries.model.Timeseries;

public class WaveformServiceTest {

	WaveformService waveFormService;
	private ServiceUtil serviceManager;

	@Before
	public void setUp() throws FileNotFoundException, IOException {
		serviceManager = ServiceUtil.getInstance();
		this.waveFormService=serviceManager.getWaveformService();
	}

	@Test//(expected = CriteriaException.class)
	public void noDataFoundException() throws Exception {
		
		String loggerName = "edu.iris";
        Logger log = Logger.getLogger(loggerName);
        ConsoleHandler handler = new ConsoleHandler();
        handler.setLevel(Level.ALL);
        log.addHandler(handler);
        log.setLevel(Level.ALL);
        log.fine("just testing");
        
        
        
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));

		Date a = dfm.parse("2003-06-01T01:00:00");

		Date b = dfm.parse("2003-06-01T01:00:0");
		// IU ANMO 00 BHZ 2010-084T00:00:00 2010-091T00:00:00
		WaveformCriteria criteria = new WaveformCriteria();
		criteria.add("ZH", "E2", "00", "BHE", a, b);
		// no network of this value, therefore 404
		System.out.println(criteria.toUrlParams());
		this.waveFormService = serviceManager.getWaveformService();
		//this.waveFormService = serviceManager.getWaveformService("http://wslive1:8081/ws/bulkdataselect/");
		this.waveFormService.fetch(criteria);
	    System.out.println(waveFormService.getBaseUrl());
		
		
		//File file = new File("/Users/yazan/bd_M_a112a25.miniseed");
		//InputStream inStream = new FileInputStream(file);
		//waveFormService.load(inStream);
		
	}

	@Test
	public void distinctWithNoDataForSome() throws Exception {
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date a = dfm.parse("2010-02-26T20:15:00");

		Date b = dfm.parse(" 2010-02-26T22:00:00");
		// IU ANMO 00 BHZ 2010-084T00:00:00 2010-091T00:00:00
		WaveformCriteria criteria = new WaveformCriteria();
		criteria.add("IU", "ANMO", "00", "BHZ", a, b);
		
		criteria.add("UU", "ANMO", "00", "BHZ", a, b).makeDistinctRequests(true);

		List<Timeseries> timeSeriesCollection = waveFormService.fetch(criteria);
		assertNotNull(timeSeriesCollection);
		assertTrue(!timeSeriesCollection.isEmpty());

		Timeseries timeseries = null;
		for (Timeseries e : timeSeriesCollection) {
			if ("IU".equalsIgnoreCase(e.getNetworkCode())) {
				timeseries = e;

				for (Segment s : e.getSegments()) {

					// System.out.println("Sample Count: "+s.getSampleCount());
					if (s.getType() == Type.INTEGER) {
						assertNotNull(s.getIntData());
						assertEquals(s.getIntData().size(), s.getSampleCount());
					} else if (s.getType() == Type.FLOAT) {
						assertNotNull(s.getFloatData());
						assertEquals(s.getFloatData().size(),
								s.getSampleCount());
					} else if (s.getType() == Type.DOUBLE) {
						assertNotNull(s.getDoubleData());
						assertEquals(s.getDoubleData().size(),
								s.getSampleCount());
					} else if (s.getType() == Type.SHORT) {
						assertNotNull(s.getShortData());
						assertEquals(s.getShortData().size(),
								s.getSampleCount());
					}
				}
			}
		}

		assertNotNull(timeseries);

		//assertEquals(162624, timeseries.getDataSize());

	}

	@Test
	public void fetchAuth() throws Exception {
		String network = "IU";
		String station = "ANMO";
		String channel = "BHZ";
		String location = "00";
		String startDateStr = "2012-02-20 00:00:00.000";
		String endDateStr = "2012-02-20 1:00:00.000";
		char quality = 'B';
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd HH:mm:sss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date a = dfm.parse(startDateStr);

		Date b = dfm.parse(endDateStr);
		try {
			waveFormService.setAuth("nobody@iris.edu", "anonymous");

			// IU ANMO 00 BHZ 2010-084T00:00:00 2010-091T00:00:00
			WaveformCriteria criteria = new WaveformCriteria();
			criteria.add(network, station, location, channel, a, b)
					.makeDistinctRequests(true);

			List<Timeseries> timeSeriesCollection = waveFormService.fetch(criteria);
			System.out.println(">>>>>>");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	@Test
	public void fetchAuthUserPass() throws Exception {
		String network = "IU";
		String station = "ANMO";
		String channel = "BHZ";
		String location = "00";
		String startDateStr = "2012-02-20 00:00:00.000";
		String endDateStr = "2012-02-20 1:00:00.000";
		char quality = 'B';
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd HH:mm:sss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date a = dfm.parse(startDateStr);

		Date b = dfm.parse(endDateStr);
		try {
			waveFormService.setAuth("nobody@iris.edu", "anonymous");

			// IU ANMO 00 BHZ 2010-084T00:00:00 2010-091T00:00:00
			WaveformCriteria criteria = new WaveformCriteria();
			criteria.add(network, station, location, channel, a, b)
					.makeDistinctRequests(true);

			List<Timeseries> timeSeriesCollection = waveFormService.fetch(criteria, "nobody@iris.edu", "anonymous");
			System.out.println(">>>>>>");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	@Test
	public void fetch() throws Exception {
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date a = dfm.parse("2010-02-26T20:15:00");

		Date b = dfm.parse(" 2010-02-26T22:00:00");
		// IU ANMO 00 BHZ 2010-084T00:00:00 2010-091T00:00:00
		WaveformCriteria criteria = new WaveformCriteria();
		criteria.add("IU", "ANMO", "00", "BHZ", a, b);

		List<Timeseries> timeSeriesCollection = waveFormService.fetch(criteria);
		assertNotNull(timeSeriesCollection);
		assertTrue(!timeSeriesCollection.isEmpty());

		Timeseries timeseries = null;
		for (Timeseries e : timeSeriesCollection) {
			if ("IU".equalsIgnoreCase(e.getNetworkCode())) {
				timeseries = e;

				for (Segment s : e.getSegments()) {

					// System.out.println("Sample Count: "+s.getSampleCount());
					if (s.getType() == Type.INTEGER) {
						assertNotNull(s.getIntData());
						assertEquals(s.getIntData().size(), s.getSampleCount());
					} else if (s.getType() == Type.FLOAT) {
						assertNotNull(s.getFloatData());
						assertEquals(s.getFloatData().size(),
								s.getSampleCount());
					} else if (s.getType() == Type.DOUBLE) {
						assertNotNull(s.getDoubleData());
						assertEquals(s.getDoubleData().size(),
								s.getSampleCount());
					} else if (s.getType() == Type.SHORT) {
						assertNotNull(s.getShortData());
						assertEquals(s.getShortData().size(),
								s.getSampleCount());
					}
				}
			}
		}

		assertNotNull(timeseries);

		//assertEquals(162624, timeseries.getDataSize());

	}

	@Test
	public void unappliedTimeCorrection() throws Exception {
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date a = dfm.parse("1996-02-17T05:42:00");

		Date b = dfm.parse("1996-2-17T05:45:00");

		WaveformCriteria criteria = new WaveformCriteria();
		criteria.add("G", "DRV", "--", "BHE", a, b);

		List<Timeseries> timeSeriesCollection = waveFormService.fetch(criteria);

		assertEquals(1, timeSeriesCollection.size());

		Timeseries timeseries = timeSeriesCollection.get(0);
		assertEquals("G", timeseries.getNetworkCode());
		//assertEquals(4032, timeseries.getDataSize());

		assertEquals(1, timeseries.getSegments().size());

		Segment segment = timeseries.getSegments().iterator().next();

		if (segment.getType() == Type.INTEGER) {
			assertNotNull(segment.getIntData());
			assertEquals(segment.getIntData().size(), segment.getSampleCount());
		} else if (segment.getType() == Type.FLOAT) {
			assertNotNull(segment.getFloatData());
			assertEquals(segment.getFloatData().size(),
					segment.getSampleCount());
		} else if (segment.getType() == Type.DOUBLE) {
			assertNotNull(segment.getDoubleData());
			assertEquals(segment.getDoubleData().size(),
					segment.getSampleCount());
		} else if (segment.getType() == Type.SHORT) {
			assertNotNull(segment.getShortData());
			assertEquals(segment.getShortData().size(),
					segment.getSampleCount());
		}

		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		cal.set(Calendar.YEAR, 1996);
		cal.set(Calendar.DAY_OF_YEAR, 48);
		cal.set(Calendar.HOUR_OF_DAY, 5);
		cal.set(Calendar.MINUTE, 42);
		cal.set(Calendar.SECOND, 19);

		Timestamp expectedStartTime = new Timestamp(cal.getTime().getTime());
		expectedStartTime.setNanos(704200000);

		cal.set(Calendar.YEAR, 1996);
		cal.set(Calendar.DAY_OF_YEAR, 48);
		cal.set(Calendar.HOUR_OF_DAY, 5);
		cal.set(Calendar.MINUTE, 44);
		cal.set(Calendar.SECOND, 56);
		Timestamp expectedEndTime = new Timestamp(cal.getTime().getTime());
		expectedEndTime.setNanos(804200000);

		assertEquals(expectedStartTime, segment.getStartTime());

		assertEquals(expectedEndTime, segment.getEndTime());

	}

	@Test
	public void appliedTimeCorrection() throws Exception {
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date a = dfm.parse("2006-09-11T00:00:00");

		Date b = dfm.parse("2006-09-11T00:01:00");
		// IU ANMO 00 BHZ 2010-084T00:00:00 2010-091T00:00:00
		WaveformCriteria criteria = new WaveformCriteria();
		criteria.add("AV", "SPBG", "--", "BHZ", a, b);

		List<Timeseries> timeSeriesCollection = waveFormService.fetch(criteria);

		for (Timeseries timeseries : timeSeriesCollection) {
			timeseries.getNetworkCode();
			timeseries.getStationCode();
			timeseries.getChannelCode();
			for (Segment segment : timeseries.getSegments()) {
				segment.getSampleCount();
				segment.getStartTime();
				segment.getEndTime();
				if (segment.getType() == Type.INTEGER) {
					segment.getIntData();
				}
				// Do something
			}
		}

		assertEquals(1, timeSeriesCollection.size());

		Timeseries timeseries = timeSeriesCollection.get(0);
		assertEquals("AV", timeseries.getNetworkCode());
		//assertEquals(1792, timeseries.getDataSize());

		dfm = new SimpleDateFormat("yyyy,D,HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));

		assertEquals(1, timeseries.getSegments().size());

		Segment segment = timeseries.getSegments().iterator().next();

		if (segment.getType() == Type.INTEGER) {
			assertNotNull(segment.getIntData());
			assertEquals(segment.getIntData().size(), segment.getSampleCount());
		} else if (segment.getType() == Type.FLOAT) {
			assertNotNull(segment.getFloatData());
			assertEquals(segment.getFloatData().size(),
					segment.getSampleCount());
		} else if (segment.getType() == Type.DOUBLE) {
			assertNotNull(segment.getDoubleData());
			assertEquals(segment.getDoubleData().size(),
					segment.getSampleCount());
		} else if (segment.getType() == Type.SHORT) {
			assertNotNull(segment.getShortData());
			assertEquals(segment.getShortData().size(),
					segment.getSampleCount());
		}

		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		cal.set(Calendar.YEAR, 2006);
		cal.set(Calendar.DAY_OF_YEAR, 254);
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);

		Timestamp expectedStartTime = new Timestamp(cal.getTime().getTime());
		expectedStartTime.setNanos(0);

		cal.set(Calendar.YEAR, 2006);
		cal.set(Calendar.DAY_OF_YEAR, 254);
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 22);
		Timestamp expectedEndTime = new Timestamp(cal.getTime().getTime());
		expectedEndTime.setNanos(0);

		assertEquals(expectedStartTime, segment.getStartTime());

		assertEquals(expectedEndTime, segment.getEndTime());
	}

	@Test
	public void load() throws Exception {
		File theFile = new File(
				"/Users/yazan/iris/test/waveFormLoadedTestFile.seed");
		InputStream inputStream = new FileInputStream(theFile);
		List<Timeseries> timeSeriesCollection = this.waveFormService
				.load(inputStream);

		assertNotNull(timeSeriesCollection);
		assertTrue(!timeSeriesCollection.isEmpty());

		Timeseries timeseries = null;
		for (Timeseries e : timeSeriesCollection) {
			if ("IU".equalsIgnoreCase(e.getNetworkCode())) {
				timeseries = e;

				for (Segment s : e.getSegments()) {

					// System.out.println("Sample Count: "+s.getSampleCount());
					if (s.getType() == Type.INTEGER) {
						assertNotNull(s.getIntData());
						assertEquals(s.getIntData().size(), s.getSampleCount());
					} else if (s.getType() == Type.FLOAT) {
						assertNotNull(s.getFloatData());
						assertEquals(s.getFloatData().size(),
								s.getSampleCount());
					} else if (s.getType() == Type.DOUBLE) {
						assertNotNull(s.getDoubleData());
						assertEquals(s.getDoubleData().size(),
								s.getSampleCount());
					} else if (s.getType() == Type.SHORT) {
						assertNotNull(s.getShortData());
						assertEquals(s.getShortData().size(),
								s.getSampleCount());
					}
				}
			}
		}

		assertNotNull(timeseries);

		//assertEquals(162624, timeseries.getDataSize());
	}

	@Test
	public void stream() throws Exception {

		File theFile = new File(
				"/Users/yazan/iris/test/waveform_test_input.seed");
		OutputStream out = new FileOutputStream(theFile);
		assertTrue(theFile.exists());
		DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date a = dfm.parse("2010-02-26T20:15:00");

		Date b = dfm.parse(" 2010-02-26T22:00:00");
		// IU ANMO 00 BHZ 2010-084T00:00:00 2010-091T00:00:00
		WaveformCriteria criteria = new WaveformCriteria();
		criteria.add("IU", "ANMO", "00", "BHZ", a, b);

		this.waveFormService.stream(out, criteria);
		out.flush();
		out.close();

		assertEquals(185856, theFile.length());

	}

}
