package edu.iris.dmc.ws;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import edu.iris.dmc.criteria.CriteriaException;
import edu.iris.dmc.criteria.WaveformCriteria;
import edu.iris.dmc.fdsn.station.model.Channel;
import edu.iris.dmc.fdsn.station.model.Network;
import edu.iris.dmc.fdsn.station.model.Station;
import edu.iris.dmc.service.NoDataFoundException;
import edu.iris.dmc.service.ServiceNotSupportedException;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.StationService;
import edu.iris.dmc.service.UnauthorizedAccessException;
import edu.iris.dmc.service.WaveformService;

public class WaveformThread1 {

	public static void main(String[] args) {
		System.setProperty("http.keepalive", "false");
		ServiceUtil serviceUtil = ServiceUtil.getInstance();

		StationService stationService = serviceUtil.getStationService();
		List<Network> networks = null;
		try {
			networks = stationService
					.fetch("http://service.adc1.iris.edu/fdsnws/station/1/query?level=cha&net=IU&sta=AN*");
		} catch (NoDataFoundException e1) {
			// TODO Auto-generated catch block
			// e1.printStackTrace();
			System.out.println("404");
		} catch (CriteriaException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (ServiceNotSupportedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		edu.iris.dmc.service.WaveformService waveFormService = serviceUtil
				.getWaveformService("http://service.adc1.iris.edu/fdsnws/dataselect/1/");

		int i = 0;
		if (networks != null) {
			loop: for (Network network : networks) {
				for (Station station : network.getStations()) {
					if (station.getChannels() != null) {
						for (Channel channel : station.getChannels()) {
							if (i > 25) {
								break loop;
							}
							GregorianCalendar cal = channel.getStartDate()
									.toGregorianCalendar();
							Date start = cal.getTime();
							cal.add(Calendar.DATE, 1);
							Date end = cal.getTime();
							WaveformCriteria criteria = new WaveformCriteria();
							criteria.add(network.getCode(), station.getCode(),
									"*",
									channel.getCode(), start, end);
							Thread t = new Thread(new WaveFormThread(
									waveFormService, criteria, "Thread: " + i));
							t.start();
							i++;
						}
					}
				}

			}
		}

	}

	private static class WaveFormThread implements Runnable {

		private WaveformService waveFormService = null;
		private WaveformCriteria waveFormCriteria = null;
		private String name;

		public WaveFormThread(WaveformService waveFormService,
				WaveformCriteria waveFormCriteria, String name) {
			this.waveFormCriteria = waveFormCriteria;
			this.waveFormService = waveFormService;
			this.name = name;
		}

		@Override
		public void run() {
			System.out.println(name);
			FileOutputStream out = null;
			try {
				out = new FileOutputStream(name+".seed");
				if (this.waveFormService == null) {

				}

				if (waveFormCriteria == null) {

				}

				this.waveFormService.stream(out, waveFormCriteria);
				System.out.println("Finished " + name);
			} catch (NoDataFoundException e) {
				// TODO Auto-generated catch block
				// e.printStackTrace();
				System.out.println(name + " 404");
			} catch (IOException e) {
				System.out.println(name);
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (CriteriaException e) {
				System.out.println(name);
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ServiceNotSupportedException e) {
				System.out.println(name);
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (UnauthorizedAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}finally{
				if(out!=null){
					try {
						out.close();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}

		}

	}

}
