package edu.iris.dmc.ws.extensions;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import edu.iris.dmc.criteria.StationCriteria;
import edu.iris.dmc.extensions.entities.Trace;
import edu.iris.dmc.extensions.fetch.TraceData;
import edu.iris.dmc.extensions.utils.SacUtil;
import edu.iris.dmc.sacpz.model.Pole;
import edu.iris.dmc.sacpz.model.Sacpz;
import edu.iris.dmc.sacpz.model.Zero;
import edu.iris.dmc.service.ServiceUtil;
import edu.iris.dmc.service.StationService;
import edu.sc.seis.seisFile.sac.SacTimeSeries;

public class SacTraceTest {

	StationService stationService;

	@Before
	public void setUp() throws FileNotFoundException, IOException {
		ServiceUtil serviceManager = ServiceUtil.getInstance();
		this.stationService = serviceManager.getStationService();
	}

	// @Test
	public void doSac() throws Exception {

		DateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

		StationCriteria stationCriteria = new StationCriteria();
		stationCriteria.addNetwork("IU").addStation("ANMO").addChannel("BHZ")
				.addLocation("00");
		Date startDate = sdf.parse("2011-10-13T00:00:00");
		Date endDate = sdf.parse("2011-10-14T00:00:00");
		stationCriteria.setStartTime(startDate);
		stationCriteria.setEndTime(endDate);

		StationCriteria[] array = new StationCriteria[1];
		array[0] = stationCriteria;
		Trace[] traces = TraceData.fetchTraces(array, new Character(' '), true);

		for (Trace trace : traces) {
			SacTimeSeries sac = SacUtil.makeSac(trace);
			String fname = SacUtil.getSacFilename(trace);
			// userMessage("Wrote " + sac.getHeader().getNpts() + " samples to "
			// + fname, true);
			Sacpz sacpz = trace.getSacpz();
			for (Zero z : sacpz.getZeros()) {
				// System.out.println(z.getReal()+"  "+z.getImaginary());
			}
			for (Pole p : sacpz.getPoles()) {
				// System.out.println(p.getReal()+"  "+p.getImaginary());
			}
			// System.out.println(sacpz.getConstant());
			// System.out.println("*****************");
			sac.write(fname);
		}
	}

	// @Test
	public void doSacWithAuth() throws Exception {

		DateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

		StationCriteria stationCriteria = new StationCriteria();
		stationCriteria.addNetwork("IU").addStation("ANMO").addChannel("BHZ")
				.addLocation("00");
		Date startDate = sdf.parse("2011-10-13T00:00:00");
		Date endDate = sdf.parse("2011-10-14T00:00:00");
		stationCriteria.setStartTime(startDate);
		stationCriteria.setEndTime(endDate);

		StationCriteria[] array = new StationCriteria[1];
		array[0] = stationCriteria;
		Trace[] traces = TraceData.fetchTraces(array, new Character(' '),
				"nobody@iris.edu", "anonymous", false);

		for (Trace trace : traces) {
			SacTimeSeries sac = SacUtil.makeSac(trace);
			String fname = SacUtil.getSacFilename(trace);
			// userMessage("Wrote " + sac.getHeader().getNpts() + " samples to "
			// + fname, true);
			sac.write(fname);
		}
	}

	@Test
	public void d() throws Exception {
		TraceData.fetchTraces("UW", "RCM", "", "EHZ", "2003-02-20 00:00:00",
				"2003-02-20 00:05:00", 'B', true);
	}
}
